(in-package cl)
(defpackage optparser-tests (:use :optparser :cl :ptester))
(in-package optparser-tests)


(defmacro test-mv (values form)
	`(testx ,values ,form :multiple-values t :test #'equal))
(defmacro testx (&rest argz)
	`(handler-case
		(test ,@argz :test #'equal)
		(t (ex)
			(test t (format nil "~s: ~a" ex ex)))))


(eval-when
	(:compile-toplevel :load-toplevel :execute)
	(defparameter *testing-specz*
		'(("v" "verbose")
			("d" "debug")
			("i" "itm" :value t)
			("a" "arg" :value 1)
			("none" :value nil))))


(defun do-tests ()
	(let ((*break-on-test-failures* nil))


		(with-tests (:name "parse-argv")
			(test-mv '(() ()) (parse-argv () ()))
			(test-mv '(("arg1" "arg2") ()) (parse-argv '("arg1" "arg2") ()))
			(test-mv '(() ((1))) (parse-argv () '(("v" :value nil :idx 1))))

			(test-mv
				'(("enc" "path1" "path2")
					(("timeout" . 10) ("t" . 10) ("timeout" . "20") (2 . t) (:conc . "10") (1 . t)))
				(parse-argv '("enc" "-vc" "10" "--logging" "path1" "--timeout=20" "path2")
					'(("v" :value nil :idx 1) ("verbose" :value nil :idx 1)
						("l" :value nil :idx 2) ("logging" :value nil :idx 2) ("log-to" :value t :idx 2)
						("c" :value t :idx :conc) ("concurrency" :value t :idx :conc)
						("t" :value 10) ("timeout" :value 10))))

			(test-mv
				'(("enc" "path")
					(("timeout" . 10) ("t" . 10) (:conc . t) (2) (1)))
				(parse-argv '("enc" "path")
					'(("v" :value nil :idx 1) ("verbose" :value nil :idx 1)
						("l" :value nil :idx 2) ("logging" :value nil :idx 2) ("log-to" :value t :idx 2)
						("c" :value t :idx :conc) ("concurrency" :value t :idx :conc)
						("t" :value 10) ("timeout" :value 10)))))


		(with-tests (:name "get-argv")
			(macrolet
				((with-spec (argv) `(get-argv ,argv ,@*testing-specz*)))

				(test-mv '(("arg1" "-") t nil t "-vi" nil)
					(with-spec '("arg1" "--arg" "-vi" "-" "--verbose")))
				(test-mv '(nil nil nil t 1 nil)
					(with-spec ()))
				(test-error (with-spec '("arg1" "-itm" "1")))
				(test-error (with-spec '("arg1" "-ia" "1")))
				(test-error (with-spec '("arg1" "-iv" "1")))
				(test-error (with-spec '("arg1" "-v" "1" "-i")))))


		(with-tests (:name "getopt")
			(macrolet
				((with-spec (argv) `(getopt ,argv ,@*testing-specz*)))

				(test-mv '(() (("none" . nil) ("a" . 1) ("i" . t) ("d" . nil) ("v" . nil))) (with-spec ()))
				(test-mv '(("arg1" "-") (("none" . nil) ("i" . t) ("d" . nil) ("v" . t) ("a" . "-vi")))
					(with-spec '("arg1" "--arg" "-vi" "-" "--verbose")))))


		(with-tests (:name "argv-bind")
			(macrolet
				((with-spec (argv)
					(let ((vals `(args ,@(loop for sym in *testing-specz* collect (gensym)))))
						`(argv-bind ,vals (,argv ,@*testing-specz*)
							(list ,@vals)))))

				(testx '(nil nil nil t 1 nil) (with-spec ()))
				(testx '(("arg1" "-") t nil t "-vi" nil) (with-spec '("arg1" "--arg" "-vi" "-" "--verbose")))))


		(with-tests (:name "argv-let")
			(macrolet
				((with-spec (argv)
					(multiple-value-bind (syms specs)
						(loop for spec in *testing-specz* and sym = (gensym)
							collect sym into syms
							collect (list sym spec) into specs
							finally (return (values syms specs)))
						`(argv-let (args ,@specs)
							,argv
							(list args ,@syms)))))

				(testx '(nil nil nil t 1 nil) (with-spec ()))
				(testx '(("arg1" "-") t nil t "-vi" nil) (with-spec '("arg1" "--arg" "-vi" "-" "--verbose")))))


		;; TODO: Add more tests

		) t)
